/* FILE: errhandlers.cc                    -*-Mode: c++-*-
 *
 * Some error handling routines that should be phased out
 * and replaced with code in the oc extension.
 * 
 * NOTICE: Please see the file ../../LICENSE
 *
 * Last modified on: $Date: 1999/02/23 05:39:21 $
 * Last modified by: $Author: dgp $
 */

#include <stdarg.h>  // For va_list declarations
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "errhandlers.h"

/* End includes */     

const char* ErrArrOutOfBounds = "Array index out of bounds";
const char* ErrBadFile        = "File error";
const char* ErrBadFileHeader  = "Bad file header";
const char* ErrBadFileOpen    = "Unable to open file";
const char* ErrInputEOF       = "Premature End-Of-File on input";
const char* ErrOutputEOF      = "Premature End-Of-File on output";
const char* ErrBadParam       = "Bad parameter";
const char* ErrDispInit       = "Unable to initialize display";
const char* ErrMisc           = "Miscellaneous error";
const char* ErrNoMem          = "Insufficent working memory";
const char* ErrOverflow       = "Arithmetic overflow";
const char* ErrProgramming    = "Programming error";

#define BigBufSize 4096
static char ErrorBuf[BigBufSize];

// Error & warning messages storage
Nb_DString MessageLocker::msg;

int MessageLocker::Append(const CHAR *fmt,...)
{
  va_list arg_ptr;
  va_start(arg_ptr,fmt);
  vsprintf(ErrorBuf,fmt,arg_ptr);
  va_end(arg_ptr);
  msg.Append(ErrorBuf,sizeof(ErrorBuf));
  return 0;
}

int MessageLocker::GetMessage( Nb_DString& _msg)
{
  _msg=msg;  // Duplicates
  return 0;
}

// Things worth dying over...
void FatalError(INT4m errorcode,
		const CHAR *membername,
		const ClassDoc &class_doc,
		const CHAR *filename,INT4m lineno,
		const CHAR *fmt,...)
{
  va_list arg_ptr;
  char *cptr;
  sprintf(ErrorBuf,
	  "Fatal error occurred in %s::%s (File %s, line %d)\n"
	  "Class revision/date: %s/%s\n"
	  "Class maintainer: %s\nERROR: ",
	  class_doc.classname,membername,filename,lineno,
	  class_doc.revision,class_doc.revdate,
	  class_doc.maintainer);
  cptr=strchr(ErrorBuf,'\0'); // Note: Return type of sprintf is OS dependent
  va_start(arg_ptr,fmt);
  vsprintf(cptr,fmt,arg_ptr);
  va_end(arg_ptr);

  ErrorBuf[sizeof(ErrorBuf)-1]='\0'; // A wee bit of safety
  TkMessage(MSG_ERROR,ErrorBuf,errorcode);

  // We shouldn't get to here, but just in case...
  exit(1);
}

// Fatal Error handler for non-class functions
void PlainError(INT4m errorcode,const CHAR *fmt,...)
{
  va_list arg_ptr;
  va_start(arg_ptr,fmt);
  vsprintf(ErrorBuf,fmt,arg_ptr);
  va_end(arg_ptr);

  ErrorBuf[sizeof(ErrorBuf)-1]='\0'; // A wee bit of safety
  TkMessage(MSG_ERROR,ErrorBuf,errorcode);

  // We shouldn't get to here, but just in case...
  exit(1);
}


// Warnings
void NonFatalError(const CHAR *membername,
		   const ClassDoc &class_doc,
		   const CHAR *filename,INT4m lineno,
		   const CHAR *fmt,...)
{
  va_list arg_ptr;
  char *cptr;
  sprintf(ErrorBuf,
	  "Nonfatal error occurred in %s::%s (File %s, line %d)\n"
	  "Class revision/date: %s/%s\n"
	  "Class maintainer: %s\nWARNING: ",
	  class_doc.classname,membername,filename,lineno,
	  class_doc.revision,class_doc.revdate,
	  class_doc.maintainer);
  cptr=strchr(ErrorBuf,'\0'); // Note: Return type of sprintf is OS dependent
  va_start(arg_ptr,fmt);
  vsprintf(cptr,fmt,arg_ptr);
  va_end(arg_ptr);

  ErrorBuf[sizeof(ErrorBuf)-1]='\0'; // A wee bit of safety
  TkMessage(MSG_WARNING,ErrorBuf,0);
}

// Informational messages.  Non-blocking.
void ClassMessage(const CHAR *membername,
		 const ClassDoc &class_doc,
		 const CHAR *filename,INT4m lineno,
		 const CHAR *fmt,...)
{
  va_list arg_ptr;
  char *cptr;
  sprintf(ErrorBuf,
	  "Message from %s::%s (File %s, line %d)\n"
	  "Class revision/date: %s/%s\n"
	  "Class maintainer: %s\n\n",
	  class_doc.classname,membername,filename,lineno,
	  class_doc.revision,class_doc.revdate,
	  class_doc.maintainer);
  cptr=strchr(ErrorBuf,'\0'); // Note: Return type of sprintf is OS dependent
  va_start(arg_ptr,fmt);
  vsprintf(cptr,fmt,arg_ptr);
  va_end(arg_ptr);

  ErrorBuf[sizeof(ErrorBuf)-1]='\0'; // A wee bit of safety
  TkMessage(MSG_INFO,ErrorBuf,0);
}

void ClassDebugMessage(const CHAR *membername,
		 const ClassDoc &class_doc,
		 const CHAR *filename,INT4m lineno,
		 const CHAR *fmt,...)
{
  va_list arg_ptr;
  char *cptr;
  sprintf(ErrorBuf,
	  "Debug info from %s::%s (File %s, line %d)\n"
	  "Class revision/date: %s/%s\n"
	  "Class maintainer: %s\n\n",
	  class_doc.classname,membername,filename,lineno,
	  class_doc.revision,class_doc.revdate,
	  class_doc.maintainer);
  cptr=strchr(ErrorBuf,'\0'); // Note: Return type of sprintf is OS dependent
  va_start(arg_ptr,fmt);
  vsprintf(cptr,fmt,arg_ptr);
  va_end(arg_ptr);

  ErrorBuf[sizeof(ErrorBuf)-1]='\0'; // A wee bit of safety
  TkMessage(MSG_DEBUG,ErrorBuf,0);
}

// Nonfatal error handler for non-class functions
void PlainWarning(const CHAR *fmt,...)
{
  va_list arg_ptr;
  va_start(arg_ptr,fmt);
  vsprintf(ErrorBuf,fmt,arg_ptr);
  va_end(arg_ptr);

  ErrorBuf[sizeof(ErrorBuf)-1]='\0'; // A wee bit of safety
  TkMessage(MSG_WARNING,ErrorBuf,0);
}

// Informational messages.  Non-blocking, should always return 0.
int Message(const CHAR *fmt,...)
{
  va_list arg_ptr;
  va_start(arg_ptr,fmt);
  vsprintf(ErrorBuf,fmt,arg_ptr);
  va_end(arg_ptr);

  ErrorBuf[sizeof(ErrorBuf)-1]='\0'; // A wee bit of safety
  return TkMessage(MSG_INFO,ErrorBuf,0);
}

// C-interface to Tcl/Tk error boxes.
INT4m TkMessage(MessageType mt,const CHAR *msg,INT4m exitcode)
{
  static char buf[BigBufSize];

  switch(mt) {
  case MSG_DEBUG:
    sprintf(buf,"Oc_Log Log {%s} debug",msg);
    break;
  case MSG_INFO:
    sprintf(buf,"Oc_Log Log {%s} info",msg);
    break;
  case MSG_WARNING:
    sprintf(buf,"Oc_Log Log {%s} warning",msg);
    break;
  case MSG_ERROR:
    sprintf(buf,"Oc_Log Log {%s} panic",msg);
    break;
  default:
    sprintf(buf,"Oc_Log Log {Unknown MessageType (%d) detected"
	    " in TkMessage(); Associated message: %s} panic",
	    int(mt),msg);
    break;
  }

  Tcl_Interp* interp=Oc_GlobalInterpreter();
  if(interp==NULL) {
    // Tcl interp not setup. 
    // Write to MessageBuffer
    strcat(buf,"\n");
    MessageLocker::Append(buf);
    if(mt==MSG_ERROR) {
      // If fatal error, dump MessageLocker to stderr and exit
      Nb_DString amsg;
      MessageLocker::GetMessage(amsg);
      fprintf(stderr,"%s\nFATAL ERROR\n",amsg.GetStr());
      fflush(stderr);
      exit(exitcode);
    }
    return -1;
  }

  Tcl_Eval(interp,buf);  // We could check for errors
  // from Tcl_Eval, but to where would we send a message???
  return atoi(Tcl_GetStringResult(interp));
}




